try:
    from importlib.metadata import version as _version
except ImportError as e:
    raise ImportError('Most likely you have python<3.8 installed. At least 3.8 is required.') from e

__version__ = _version('namespacelib')



from rdflib.namespace import DefinedNamespace, Namespace
from rdflib.term import URIRef


class LanguageExtension:
    pass

class PIVMETA(DefinedNamespace):
    # uri = "https://w3id.org/nfdi4ing/metadata4ing#"
    # Generated with namespacelib version 0.3.0
    # Date: 2024-03-02 13:35:41.244037
    _fail = True
    time: URIRef  # ['time']
    BackgroundImageGeneration: URIRef  # ['background image generation']
    BackgroundSubtractionMethod: URIRef  # ['background subtraction']
    CorrelationMethod: URIRef  # ['correlation method']
    DigitalCamera: URIRef  # ['Digital Camera']
    ImageDewarp: URIRef  # ['image dewarp']
    ImageFilter: URIRef  # ['image filter']
    ImageRotation: URIRef  # ['image rotation']
    ImageType: URIRef  # ['image type']
    InterrogationMethod: URIRef  # ['interrogation method']
    MaskGeneration: URIRef  # ['mask generation']
    Multigrid: URIRef  # ['multi grid']
    Multipass: URIRef  # ['multi pass']
    OutlierDetectionMethod: URIRef  # ['outlier detection method']
    OutlierReplacementScheme: URIRef  # ['outlier replacement scheme']
    PIVEvaluation: URIRef  # ['piv evaluation']
    PIVSoftware: URIRef  # ['Piv Software']
    PeakSearchMethod: URIRef  # ['peak search method']
    PivDistribution: URIRef  # ['piv distribution']
    PivImageDistribution: URIRef  # ['piv image distribution']
    PivMaskDistribution: URIRef  # ['piv mask distribution']
    PivPostProcessing: URIRef  # ['piv post processing']
    PivPreProcessing: URIRef  # ['piv pre processing']
    PivProcessingStep: URIRef  # ['piv processing step']
    PivResultDistribution: URIRef  # ['piv result distribution']
    Singlepass: URIRef  # ['single pass']
    hasStandardName: URIRef  # ['has standard name']
    pivImageType: URIRef  # ['piv image type']
    useOutlierReplacementScheme: URIRef  # ['uses outlier replacement scheme']
    filenamePattern: URIRef  # ['filename pattern']
    fnumber: URIRef  # ['fnumber']
    numberOfRecords: URIRef  # ['number of records']
    ExperimentalImage: URIRef  # ['experimental image']
    Interpolation: URIRef  # ['interpolation']
    LeftRightFlip: URIRef  # ['left right flip']
    ReEvaluateWithLargerSample: URIRef  # ['re-evaluate with larger sample']
    SplitImage: URIRef  # ['split image']
    SyntheticImage: URIRef  # ['synthetic image']
    TopBottomFlip: URIRef  # ['top bottom flip']
    TryLowerOrderPeaks: URIRef  # ['try lower order peaks']
    micro_piv: URIRef  # ['micro piv']
    piv: URIRef  # ['piv']
    piv_first_peak_correlation_coefficient: URIRef  # ['piv_first_peak_correlation_coefficient']
    ptv: URIRef  # ['ptv']
    piv_correlation_coefficient: URIRef  # ['piv_correlation_coefficient']
    piv_second_peak_correlation_coefficient: URIRef  # ['piv_second_peak_correlation_coefficient']
    piv_third_peak_correlation_coefficient: URIRef  # ['piv_third_peak_correlation_coefficient']
    first_peak_x_displacement: URIRef  # ['first_peak_x_displacement']
    first_peak_y_displacement: URIRef  # ['first_peak_y_displacement']
    piv_flag: URIRef  # ['piv_flag']
    image_particle_density: URIRef  # ['image_particle_density']
    image_density: URIRef  # ['image_density']
    image_particle_diameter: URIRef  # ['image_particle_diameter']
    piv_image_index: URIRef  # ['piv_image_index']
    inplane_velocity: URIRef  # ['inplane_velocity']
    laser_sheet_thickness: URIRef  # ['laser_sheet_thickness']
    laser_sheet_shape_factor: URIRef  # ['laser_sheet_shape_factor']
    mean_seeding_particle_size: URIRef  # ['mean_seeding_particle_size']
    number_of_particles: URIRef  # ['number_of_particles']
    laser_pulse_delay: URIRef  # ['laser_pulse_delay']
    saturated_pixel_number: URIRef  # ['saturated_pixel_number']
    second_peak_x_displacement: URIRef  # ['second_peak_x_displacement']
    second_peak_y_displacement: URIRef  # ['second_peak_y_displacement']
    seeding_material: URIRef  # ['seeding_material']
    standard_deviation_of_image_particle_diameter: URIRef  # ['standard_deviation_of_image_particle_diameter']
    third_peak_x_displacement: URIRef  # ['third_peak_x_displacement']
    third_peak_y_displacement: URIRef  # ['third_peak_y_displacement']
    relative_time: URIRef  # ['relative_time']
    velocity: URIRef  # ['velocity']
    displacement: URIRef  # ['displacement']
    x_final_interrogation_window_overlap_size: URIRef  # ['x_final_interrogation_window_overlap_size']
    x_final_interrogation_window_size: URIRef  # ['x_final_interrogation_window_size']
    x_initial_interrogation_window_size: URIRef  # ['x_initial_interrogation_window_size']
    x_pixel_coordinate: URIRef  # ['x_pixel_coordinate']
    y_final_interrogation_window_overlap_size: URIRef  # ['y_final_interrogation_window_overlap_size']
    y_final_interrogation_window_size: URIRef  # ['y_final_interrogation_window_size']
    y_initial_interrogation_window_size: URIRef  # ['y_initial_interrogation_window_size']
    y_pixel_coordinate: URIRef  # ['y_pixel_coordinate']
    pixel_origin: URIRef  # ['pixel_origin']
    coordinate: URIRef  # ['coordinate']
    signal_to_noise_ratio: URIRef  # ['signal_to_noise_ratio']
    least_square_residual_of_z_displacement_reconstruction: URIRef  # ['least_square_residual_of_z_displacement_reconstruction']
    piv_scaling_factor: URIRef  # ['piv_scaling_factor']
    image_filter_kernel_size: URIRef  # ['image_filter_kernel_size']
    ccd_width: URIRef  # ['ccd_width']
    ccd_height: URIRef  # ['ccd_height']
    focal_length: URIRef  # ['focal_length']
    sensor_pixel_width: URIRef  # ['sensor_pixel_width']
    sensor_pixel_height: URIRef  # ['sensor_pixel_height']
    sensor_pixel_width_fill_factor: URIRef  # ['sensor_pixel_width_fill_factor']
    sensor_pixel_height_fill_factor: URIRef  # ['sensor_pixel_height_fill_factor']
    image_bit_depth: URIRef  # ['image_bit_depth']

    _NS = Namespace("https://matthiasprobst.github.io/pivmeta#")

setattr(PIVMETA, "time", PIVMETA.time)
setattr(PIVMETA, "background_image_generation", PIVMETA.BackgroundImageGeneration)
setattr(PIVMETA, "background_subtraction", PIVMETA.BackgroundSubtractionMethod)
setattr(PIVMETA, "correlation_method", PIVMETA.CorrelationMethod)
setattr(PIVMETA, "Digital_Camera", PIVMETA.DigitalCamera)
setattr(PIVMETA, "image_dewarp", PIVMETA.ImageDewarp)
setattr(PIVMETA, "image_filter", PIVMETA.ImageFilter)
setattr(PIVMETA, "image_rotation", PIVMETA.ImageRotation)
setattr(PIVMETA, "image_type", PIVMETA.ImageType)
setattr(PIVMETA, "interrogation_method", PIVMETA.InterrogationMethod)
setattr(PIVMETA, "mask_generation", PIVMETA.MaskGeneration)
setattr(PIVMETA, "multi_grid", PIVMETA.Multigrid)
setattr(PIVMETA, "multi_pass", PIVMETA.Multipass)
setattr(PIVMETA, "outlier_detection_method", PIVMETA.OutlierDetectionMethod)
setattr(PIVMETA, "outlier_replacement_scheme", PIVMETA.OutlierReplacementScheme)
setattr(PIVMETA, "piv_evaluation", PIVMETA.PIVEvaluation)
setattr(PIVMETA, "Piv_Software", PIVMETA.PIVSoftware)
setattr(PIVMETA, "peak_search_method", PIVMETA.PeakSearchMethod)
setattr(PIVMETA, "piv_distribution", PIVMETA.PivDistribution)
setattr(PIVMETA, "piv_image_distribution", PIVMETA.PivImageDistribution)
setattr(PIVMETA, "piv_mask_distribution", PIVMETA.PivMaskDistribution)
setattr(PIVMETA, "piv_post_processing", PIVMETA.PivPostProcessing)
setattr(PIVMETA, "piv_pre_processing", PIVMETA.PivPreProcessing)
setattr(PIVMETA, "piv_processing_step", PIVMETA.PivProcessingStep)
setattr(PIVMETA, "piv_result_distribution", PIVMETA.PivResultDistribution)
setattr(PIVMETA, "single_pass", PIVMETA.Singlepass)
setattr(PIVMETA, "has_standard_name", PIVMETA.hasStandardName)
setattr(PIVMETA, "piv_image_type", PIVMETA.pivImageType)
setattr(PIVMETA, "uses_outlier_replacement_scheme", PIVMETA.useOutlierReplacementScheme)
setattr(PIVMETA, "filename_pattern", PIVMETA.filenamePattern)
setattr(PIVMETA, "fnumber", PIVMETA.fnumber)
setattr(PIVMETA, "number_of_records", PIVMETA.numberOfRecords)
setattr(PIVMETA, "experimental_image", PIVMETA.ExperimentalImage)
setattr(PIVMETA, "interpolation", PIVMETA.Interpolation)
setattr(PIVMETA, "left_right_flip", PIVMETA.LeftRightFlip)
setattr(PIVMETA, "re-evaluate_with_larger_sample", PIVMETA.ReEvaluateWithLargerSample)
setattr(PIVMETA, "split_image", PIVMETA.SplitImage)
setattr(PIVMETA, "synthetic_image", PIVMETA.SyntheticImage)
setattr(PIVMETA, "top_bottom_flip", PIVMETA.TopBottomFlip)
setattr(PIVMETA, "try_lower_order_peaks", PIVMETA.TryLowerOrderPeaks)
setattr(PIVMETA, "micro_piv", PIVMETA.micro_piv)
setattr(PIVMETA, "piv", PIVMETA.piv)
setattr(PIVMETA, "piv_first_peak_correlation_coefficient", PIVMETA.piv_first_peak_correlation_coefficient)
setattr(PIVMETA, "ptv", PIVMETA.ptv)
setattr(PIVMETA, "piv_correlation_coefficient", PIVMETA.piv_correlation_coefficient)
setattr(PIVMETA, "piv_second_peak_correlation_coefficient", PIVMETA.piv_second_peak_correlation_coefficient)
setattr(PIVMETA, "piv_third_peak_correlation_coefficient", PIVMETA.piv_third_peak_correlation_coefficient)
setattr(PIVMETA, "first_peak_x_displacement", PIVMETA.first_peak_x_displacement)
setattr(PIVMETA, "first_peak_y_displacement", PIVMETA.first_peak_y_displacement)
setattr(PIVMETA, "piv_flag", PIVMETA.piv_flag)
setattr(PIVMETA, "image_particle_density", PIVMETA.image_particle_density)
setattr(PIVMETA, "image_density", PIVMETA.image_density)
setattr(PIVMETA, "image_particle_diameter", PIVMETA.image_particle_diameter)
setattr(PIVMETA, "piv_image_index", PIVMETA.piv_image_index)
setattr(PIVMETA, "inplane_velocity", PIVMETA.inplane_velocity)
setattr(PIVMETA, "laser_sheet_thickness", PIVMETA.laser_sheet_thickness)
setattr(PIVMETA, "laser_sheet_shape_factor", PIVMETA.laser_sheet_shape_factor)
setattr(PIVMETA, "mean_seeding_particle_size", PIVMETA.mean_seeding_particle_size)
setattr(PIVMETA, "number_of_particles", PIVMETA.number_of_particles)
setattr(PIVMETA, "laser_pulse_delay", PIVMETA.laser_pulse_delay)
setattr(PIVMETA, "saturated_pixel_number", PIVMETA.saturated_pixel_number)
setattr(PIVMETA, "second_peak_x_displacement", PIVMETA.second_peak_x_displacement)
setattr(PIVMETA, "second_peak_y_displacement", PIVMETA.second_peak_y_displacement)
setattr(PIVMETA, "seeding_material", PIVMETA.seeding_material)
setattr(PIVMETA, "standard_deviation_of_image_particle_diameter", PIVMETA.standard_deviation_of_image_particle_diameter)
setattr(PIVMETA, "third_peak_x_displacement", PIVMETA.third_peak_x_displacement)
setattr(PIVMETA, "third_peak_y_displacement", PIVMETA.third_peak_y_displacement)
setattr(PIVMETA, "relative_time", PIVMETA.relative_time)
setattr(PIVMETA, "velocity", PIVMETA.velocity)
setattr(PIVMETA, "displacement", PIVMETA.displacement)
setattr(PIVMETA, "x_final_interrogation_window_overlap_size", PIVMETA.x_final_interrogation_window_overlap_size)
setattr(PIVMETA, "x_final_interrogation_window_size", PIVMETA.x_final_interrogation_window_size)
setattr(PIVMETA, "x_initial_interrogation_window_size", PIVMETA.x_initial_interrogation_window_size)
setattr(PIVMETA, "x_pixel_coordinate", PIVMETA.x_pixel_coordinate)
setattr(PIVMETA, "y_final_interrogation_window_overlap_size", PIVMETA.y_final_interrogation_window_overlap_size)
setattr(PIVMETA, "y_final_interrogation_window_size", PIVMETA.y_final_interrogation_window_size)
setattr(PIVMETA, "y_initial_interrogation_window_size", PIVMETA.y_initial_interrogation_window_size)
setattr(PIVMETA, "y_pixel_coordinate", PIVMETA.y_pixel_coordinate)
setattr(PIVMETA, "pixel_origin", PIVMETA.pixel_origin)
setattr(PIVMETA, "coordinate", PIVMETA.coordinate)
setattr(PIVMETA, "signal_to_noise_ratio", PIVMETA.signal_to_noise_ratio)
setattr(PIVMETA, "least_square_residual_of_z_displacement_reconstruction", PIVMETA.least_square_residual_of_z_displacement_reconstruction)
setattr(PIVMETA, "piv_scaling_factor", PIVMETA.piv_scaling_factor)
setattr(PIVMETA, "image_filter_kernel_size", PIVMETA.image_filter_kernel_size)
setattr(PIVMETA, "ccd_width", PIVMETA.ccd_width)
setattr(PIVMETA, "ccd_height", PIVMETA.ccd_height)
setattr(PIVMETA, "focal_length", PIVMETA.focal_length)
setattr(PIVMETA, "sensor_pixel_width", PIVMETA.sensor_pixel_width)
setattr(PIVMETA, "sensor_pixel_height", PIVMETA.sensor_pixel_height)
setattr(PIVMETA, "sensor_pixel_width_fill_factor", PIVMETA.sensor_pixel_width_fill_factor)
setattr(PIVMETA, "sensor_pixel_height_fill_factor", PIVMETA.sensor_pixel_height_fill_factor)
setattr(PIVMETA, "image_bit_depth", PIVMETA.image_bit_depth)


from rdflib.namespace import DefinedNamespace, Namespace
from rdflib.term import URIRef


class LanguageExtension:
    pass

class OBO(DefinedNamespace):
    # uri = "https://w3id.org/nfdi4ing/metadata4ing#"
    # Generated with namespacelib version 0.3.0
    # Date: 2024-03-02 13:33:47.083605
    _fail = True
    BFO_0000015: URIRef  # ['Prozess', 'process']
    BFO_0000017: URIRef  # ['realisierbare Entität', 'realizable entity']
    BFO_0000050: URIRef  # ['Teil von', 'part of']
    BFO_0000051: URIRef  # ['has part', 'hat Teil']
    BFO_0000054: URIRef  # ['realisiert in', 'realized in']
    BFO_0000055: URIRef  # ['realisiert', 'realizes']
    BFO_0000063: URIRef  # ['ist Voraussetzung für Schritt', 'precedes']
    RO_0000056: URIRef  # ['ist beteiligt an', 'participates in']
    RO_0000057: URIRef  # ['has participant', 'hat Teilnehmer']
    RO_0002090: URIRef  # ['immediately precedes', 'ist unmittelbare Voraussetzung für Schritt']
    RO_0002224: URIRef  # ['beginnt mit', 'starts with']
    RO_0002230: URIRef  # ['endet mit', 'ends with']
    RO_0002233: URIRef  # ['has input', 'hat Input']
    RO_0002234: URIRef  # ['has output', 'hat Output']
    RO_0002352: URIRef  # ['Input von', 'input of']
    RO_0002353: URIRef  # ['Output von', 'output of']

    _NS = Namespace("http://purl.obolibrary.org/obo/")

de = LanguageExtension()

setattr(de, "Prozess", OBO.BFO_0000015)
setattr(OBO, "process", OBO.BFO_0000015)
setattr(de, "realisierbare_Entität", OBO.BFO_0000017)
setattr(OBO, "realizable_entity", OBO.BFO_0000017)
setattr(de, "Teil_von", OBO.BFO_0000050)
setattr(OBO, "part_of", OBO.BFO_0000050)
setattr(OBO, "has_part", OBO.BFO_0000051)
setattr(de, "hat_Teil", OBO.BFO_0000051)
setattr(de, "realisiert_in", OBO.BFO_0000054)
setattr(OBO, "realized_in", OBO.BFO_0000054)
setattr(de, "realisiert", OBO.BFO_0000055)
setattr(OBO, "realizes", OBO.BFO_0000055)
setattr(de, "ist_Voraussetzung_für_Schritt", OBO.BFO_0000063)
setattr(OBO, "precedes", OBO.BFO_0000063)
setattr(de, "ist_beteiligt_an", OBO.RO_0000056)
setattr(OBO, "participates_in", OBO.RO_0000056)
setattr(OBO, "has_participant", OBO.RO_0000057)
setattr(de, "hat_Teilnehmer", OBO.RO_0000057)
setattr(OBO, "immediately_precedes", OBO.RO_0002090)
setattr(de, "ist_unmittelbare_Voraussetzung_für_Schritt", OBO.RO_0002090)
setattr(de, "beginnt_mit", OBO.RO_0002224)
setattr(OBO, "starts_with", OBO.RO_0002224)
setattr(de, "endet_mit", OBO.RO_0002230)
setattr(OBO, "ends_with", OBO.RO_0002230)
setattr(OBO, "has_input", OBO.RO_0002233)
setattr(de, "hat_Input", OBO.RO_0002233)
setattr(OBO, "has_output", OBO.RO_0002234)
setattr(de, "hat_Output", OBO.RO_0002234)
setattr(de, "Input_von", OBO.RO_0002352)
setattr(OBO, "input_of", OBO.RO_0002352)
setattr(de, "Output_von", OBO.RO_0002353)
setattr(OBO, "output_of", OBO.RO_0002353)

setattr(OBO, "de", de)


"""utility to automatically write namespaces python files automatically"""
import datetime
import json
import pathlib
import requests
import warnings
from rdflib import Graph
from typing import Iterable, Dict, Union

from . import __version__

__this_dir__ = pathlib.Path(__file__).parent


def generate_namespace_file_from_ttl(namespace: str,
                                     source: str,
                                     target_dir: Union[str, pathlib.Path, None] = None,
                                     fail=True):
    """Generate M4I_NAMESPACE.py file from m4i_context.jsonld
    """
    g = Graph()
    g.parse(source)

    if target_dir is None:
        target_dir = __this_dir__
    else:
        target_dir = pathlib.Path(target_dir)

    with open(target_dir / f'{namespace}.py', 'w',
              encoding='UTF8') as f:
        f.write('from rdflib.namespace import DefinedNamespace, Namespace\n')
        f.write('from rdflib.term import URIRef\n\n\n')
        f.write(f'class {namespace.upper()}(DefinedNamespace):')
        f.write(f'\n    # Generated with {__package__} version {__version__}')
        f.write(f'\n    _fail = {fail}')

        for s in g.subjects():
            u = str(s).rsplit('/', 1)[-1].replace('-', '_')
            if '#' in u:
                warnings.warn(f'Skipping {u} ({s}) because it has a "#" in it.')
            elif u[0].isdigit():
                warnings.warn(f'Skipping {u} ({s}) because it starts with a digit.')
            elif u in ('True', 'False', 'yield'):
                warnings.warn(f'Skipping {u} ({s}) because it starts with "yield".')
            else:
                uri = str(s)
                f.write(f'\n    {u} = URIRef("{uri}")')

        f.write('\n\n    _NS = Namespace("https://qudt.org/vocab/unit/")')

        # f.write('\n\n')
        # f.write('\n\nQUDT_UNIT = _QUDT_UNIT()')


def generate_namespace_file_from_context(namespace: str,
                                         context_ld: str,
                                         languages: Dict[str, Iterable[str]] = None,
                                         target_dir: Union[str, pathlib.Path, None] = None,
                                         fail=True):
    """Generate M4I_NAMESPACE.py file from m4i_context.jsonld"""
    languages = languages or {}
    assert isinstance(languages, dict)
    context_file = __this_dir__ / f'{namespace}.jsonld'
    if not context_file.exists():
        with open(context_file, 'w', encoding='utf-8') as f:
            f.write(requests.get(context_ld).text, )

    # read context file:
    with open(context_file, encoding='utf-8') as f:
        context = json.load(f)

    url = context['@context'][namespace]

    iris = {}
    for k, v in context['@context'].items():
        if k not in ('type', 'id'):
            if '@id' in v:
                if namespace in v['@id']:
                    name = v["@id"].rsplit(":", 1)[-1]

                    if '#' in name:
                        warnings.warn(f'Skipping {name} ({v}) because it has a "#" in it.')
                    elif name[0].isdigit():
                        warnings.warn(f'Skipping {name} ({v}) because it starts with a digit.')
                    elif name in ('True', 'False', 'yield'):
                        warnings.warn(f'Skipping {name} ({v}) because it starts with "yield".')
                    else:
                        if name not in iris:
                            iris[name] = {'url': f'{url}{name}', 'keys': [k, ]}
                        else:
                            iris[name]['keys'].append(k)
    if target_dir is None:
        target_dir = __this_dir__
    else:
        target_dir = pathlib.Path(target_dir)

    with open(target_dir / f'{namespace}.py', 'w',
              encoding='UTF8') as f:
        f.write('from rdflib.namespace import DefinedNamespace, Namespace\n')
        f.write('from rdflib.term import URIRef\n')
        f.write('\n\nclass LanguageExtension:\n    pass')
        f.write(f'\n\nclass {namespace.upper()}(DefinedNamespace):')
        f.write('\n    # uri = "https://w3id.org/nfdi4ing/metadata4ing#"')
        f.write(f'\n    # Generated with {__package__} version {__version__}')
        f.write(f'\n    # Date: {datetime.datetime.now()}')
        f.write(f'\n    _fail = {fail}')
        for k, v in iris.items():
            f.write(f'\n    {k}: URIRef  # {v["keys"]}')

        f.write(f'\n\n    _NS = Namespace("{url}")')

        for lang in languages:
            f.write(f'\n\n{lang} = LanguageExtension()')

        f.write('\n')

        for k, v in iris.items():
            for kk in v["keys"]:
                found_language_key = False
                key = kk.replace(' ', '_')
                for lang_key, lang_values in languages.items():
                    if key in lang_values:
                        f.write(f'\nsetattr({lang_key}, "{key}", {namespace.upper()}.{k})')
                        found_language_key = True
                if not found_language_key:
                    f.write(f'\nsetattr({namespace.upper()}, "{key}", {namespace.upper()}.{k})')

        for lang in languages:
            f.write(f'\n\nsetattr({namespace.upper()}, "{lang}", {lang})')
    # context_file.unlink()



from rdflib.namespace import DefinedNamespace, Namespace
from rdflib.term import URIRef


class LanguageExtension:
    pass

class SSNO(DefinedNamespace):
    # uri = "https://w3id.org/nfdi4ing/metadata4ing#"
    # Generated with namespacelib version 0.3.0
    # Date: 2024-03-02 13:35:55.978159
    _fail = True
    StandardName: URIRef  # ['StandardName']
    StandardNameTable: URIRef  # ['StandardNameTable']
    contact: URIRef  # ['contact']
    definedInStandardNameTable: URIRef  # ['standard name table']
    has_standard_names: URIRef  # ['standard names']
    quantityKind: URIRef  # ['kind of quantity']
    unit: URIRef  # ['canonical units']
    hasDOI: URIRef  # ['has doi']
    latexSymbol: URIRef  # ['has latex symbol']
    standard_name: URIRef  # ['standard name']

    _NS = Namespace("https://matthiasprobst.github.io/ssno#")

setattr(SSNO, "StandardName", SSNO.StandardName)
setattr(SSNO, "StandardNameTable", SSNO.StandardNameTable)
setattr(SSNO, "contact", SSNO.contact)
setattr(SSNO, "standard_name_table", SSNO.definedInStandardNameTable)
setattr(SSNO, "standard_names", SSNO.has_standard_names)
setattr(SSNO, "kind_of_quantity", SSNO.quantityKind)
setattr(SSNO, "canonical_units", SSNO.unit)
setattr(SSNO, "has_doi", SSNO.hasDOI)
setattr(SSNO, "has_latex_symbol", SSNO.latexSymbol)
setattr(SSNO, "standard_name", SSNO.standard_name)


from rdflib.namespace import DefinedNamespace, Namespace
from rdflib.term import URIRef


class LanguageExtension:
    pass

class CODEMETA(DefinedNamespace):
    # uri = "https://w3id.org/nfdi4ing/metadata4ing#"
    # Generated with namespacelib version 0.3.0
    # Date: 2024-03-02 13:35:43.964145
    _fail = True
    softwareSuggestions: URIRef  # ['softwareSuggestions']
    contIntegration: URIRef  # ['contIntegration']
    buildInstructions: URIRef  # ['buildInstructions']
    developmentStatus: URIRef  # ['developmentStatus']
    embargoDate: URIRef  # ['embargoDate']
    funding: URIRef  # ['funding']
    readme: URIRef  # ['readme']
    issueTracker: URIRef  # ['issueTracker']
    referencePublication: URIRef  # ['referencePublication']
    maintainer: URIRef  # ['maintainer']

    _NS = Namespace("https://codemeta.github.io/terms/")

setattr(CODEMETA, "softwareSuggestions", CODEMETA.softwareSuggestions)
setattr(CODEMETA, "contIntegration", CODEMETA.contIntegration)
setattr(CODEMETA, "buildInstructions", CODEMETA.buildInstructions)
setattr(CODEMETA, "developmentStatus", CODEMETA.developmentStatus)
setattr(CODEMETA, "embargoDate", CODEMETA.embargoDate)
setattr(CODEMETA, "funding", CODEMETA.funding)
setattr(CODEMETA, "readme", CODEMETA.readme)
setattr(CODEMETA, "issueTracker", CODEMETA.issueTracker)
setattr(CODEMETA, "referencePublication", CODEMETA.referencePublication)
setattr(CODEMETA, "maintainer", CODEMETA.maintainer)


from rdflib.namespace import DefinedNamespace, Namespace
from rdflib.term import URIRef


class LanguageExtension:
    pass

class M4I(DefinedNamespace):
    # uri = "https://w3id.org/nfdi4ing/metadata4ing#"
    # Generated with namespacelib version 0.3.0
    # Date: 2024-03-02 13:33:42.436694
    _fail = True
    Method: URIRef  # ['Methode', 'method']
    NumericalAssignment: URIRef  # ['numerical assignment', 'numerische Zuweisung']
    NumericalVariable: URIRef  # ['numerical variable', 'numerische Variable']
    ProcessingStep: URIRef  # ['Arbeitsschritt', 'processing step']
    TextVariable: URIRef  # ['text variable', 'textbasierte Variable']
    Tool: URIRef  # ['Werkzeug', 'tool']
    UncertaintyDeclaration: URIRef  # ['Unsicherheitsdeklaration', 'uncertainty declaration']
    hasAdmissibleUnit: URIRef  # ['has admissible unit', 'hat als zulässige Einheit']
    hasAdmissibleValue: URIRef  # ['has admissible value', 'hat als zulässigen Wert']
    hasAssignedValue: URIRef  # ['has assigned value', 'hat zugewiesenen Wert']
    hasCoverageInterval: URIRef  # ['has coverage interval', 'hat Überdeckungsintervall']
    hasEmployedTool: URIRef  # ['has employed tool', 'hat eingesetztes Werkzeug']
    hasExpandedUnc: URIRef  # ['has expanded uncertainty', 'hat erweiterte Unsicherheit']
    hasKindOfQuantity: URIRef  # ['has kind of quantity', 'hat Größenart']
    hasParameter: URIRef  # ['has parameter', 'hat Parameter']
    hasRuntimeAssignment: URIRef  # ['has runtime assignment', 'hat Laufzeitzuweisung']
    hasUncertaintyDeclaration: URIRef  # ['has uncertainty declaration', 'hat Unsicherheitsdeklaration']
    hasUnit: URIRef  # ['has unit', 'hat Einheit']
    hasVariable: URIRef  # ['has variable', 'hat Variable']
    inProject: URIRef  # ['associated to project', 'gehört zu Projekt']
    investigates: URIRef  # ['investigates', 'untersucht']
    investigatesProperty: URIRef  # ['investigates property', 'untersucht Eigenschaft']
    isEmployedToolIn: URIRef  # ['is employed tool', 'ist eingesetztes Werkzeug']
    projectParticipant: URIRef  # ['hat Projektmitglied', 'project participant']
    realizesMethod: URIRef  # ['realisiert Methode', 'realizes method']
    UsageInstruction: URIRef  # ['Verwendungshinweis', 'usage instruction']
    endOfProject: URIRef  # ['Projektenddatum', 'project end date']
    hasAssignmentTimestamp: URIRef  # ['has assignment timestamp', 'hat Zuweisungszeitstempel']
    hasDateAssignmentCreated: URIRef  # ['has date assignment created', 'hat Datumszuweisung erzeugt']
    hasDateAssignmentDeleted: URIRef  # ['has date assignment deleted', 'hat Datumszuweisung gelöscht']
    hasDateAssignmentModified: URIRef  # ['has date assignment modified', 'hat Datumszuweisung bearbeitet']
    hasDateAssignmentValidFrom: URIRef  # ['has date assignment valid from', 'hat Datumszuweisung gültig ab']
    hasDateAssignmentValidUntil: URIRef  # ['has date assignment valid until', 'hat Datumszuweisung gültig bis']
    hasMaximumValue: URIRef  # ['has maximum value', 'hat Maximalwert']
    hasMinimumValue: URIRef  # ['has minimum value', 'hat Minimalwert']
    hasNumericalValue: URIRef  # ['has numerical value', 'hat Zahlenwert']
    hasRorId: URIRef  # ['has ROR ID', 'hat ROR ID']
    hasStepSize: URIRef  # ['has step size', 'hat Schrittweite']
    hasStringValue: URIRef  # ['has string value', 'hat Zeichenwert']
    hasSymbol: URIRef  # ['has symbol', 'hat Symbol']
    hasValue: URIRef  # ['has value', 'hat Wert']
    hasVariableDescription: URIRef  # ['has variable description', 'hat Variablenbeschreibung']
    identifier: URIRef  # ['has identifier', 'hat Identifikator']
    orcidId: URIRef  # ['has ORCID ID', 'hat ORCID ID']
    projectReferenceID: URIRef  # ['has project ID', 'hat Projekt-ID']
    startOfProject: URIRef  # ['Projektstartdatum', 'project start date']
    ContactPerson: URIRef  # ['Kontaktperson', 'contact person']
    DataCollector: URIRef  # ['Datenerfasser*in', 'data collector']
    DataCurator: URIRef  # ['Datenkurator*in', 'data curator']
    DataManager: URIRef  # ['Datenverwalter*in', 'data manager']
    Distributor: URIRef  # ['Anbieter*in', 'distributor']
    Editor: URIRef  # ['Herausgeber*in', 'editor']
    HostingInstitution: URIRef  # ['bereitstellende Institution', 'hosting institution']
    Other: URIRef  # ['other person', 'weitere Person']
    Producer: URIRef  # ['Produzent*in', 'producer']
    ProjectLeader: URIRef  # ['Projektleiter*in', 'project leader']
    ProjectManager: URIRef  # ['Projektmanager*in', 'project manager']
    ProjectMember: URIRef  # ['Projektmitglied', 'project member']
    RegistrationAgency: URIRef  # ['Registrierungsstelle', 'registration agency']
    RegistrationAuthority: URIRef  # ['Registrierungsbehörde', 'registration authority']
    RelatedPerson: URIRef  # ['related person', 'zugehörige Person']
    ResearchGroup: URIRef  # ['Forschungsgruppe', 'research group']
    Researcher: URIRef  # ['Rechercheur*in', 'researcher']
    RightsHolder: URIRef  # ['Rechteinhaber*in', 'rights holder']
    Sponsor: URIRef  # ['Sponsor*in', 'sponsor']
    Supervisor: URIRef  # ['Betreuer*in', 'supervisor']
    WorkPackageLeader: URIRef  # ['Arbeitspaketleiter*in', 'work package leader']

    _NS = Namespace("http://w3id.org/nfdi4ing/metadata4ing#")

de = LanguageExtension()

setattr(de, "Methode", M4I.Method)
setattr(M4I, "method", M4I.Method)
setattr(M4I, "numerical_assignment", M4I.NumericalAssignment)
setattr(M4I, "numerische_Zuweisung", M4I.NumericalAssignment)
setattr(M4I, "numerical_variable", M4I.NumericalVariable)
setattr(de, "numerische_Variable", M4I.NumericalVariable)
setattr(de, "Arbeitsschritt", M4I.ProcessingStep)
setattr(M4I, "processing_step", M4I.ProcessingStep)
setattr(M4I, "text_variable", M4I.TextVariable)
setattr(de, "textbasierte_Variable", M4I.TextVariable)
setattr(de, "Werkzeug", M4I.Tool)
setattr(M4I, "tool", M4I.Tool)
setattr(de, "Unsicherheitsdeklaration", M4I.UncertaintyDeclaration)
setattr(M4I, "uncertainty_declaration", M4I.UncertaintyDeclaration)
setattr(M4I, "has_admissible_unit", M4I.hasAdmissibleUnit)
setattr(de, "hat_als_zulässige_Einheit", M4I.hasAdmissibleUnit)
setattr(M4I, "has_admissible_value", M4I.hasAdmissibleValue)
setattr(de, "hat_als_zulässigen_Wert", M4I.hasAdmissibleValue)
setattr(M4I, "has_assigned_value", M4I.hasAssignedValue)
setattr(de, "hat_zugewiesenen_Wert", M4I.hasAssignedValue)
setattr(M4I, "has_coverage_interval", M4I.hasCoverageInterval)
setattr(de, "hat_Überdeckungsintervall", M4I.hasCoverageInterval)
setattr(M4I, "has_employed_tool", M4I.hasEmployedTool)
setattr(de, "hat_eingesetztes_Werkzeug", M4I.hasEmployedTool)
setattr(M4I, "has_expanded_uncertainty", M4I.hasExpandedUnc)
setattr(de, "hat_erweiterte_Unsicherheit", M4I.hasExpandedUnc)
setattr(M4I, "has_kind_of_quantity", M4I.hasKindOfQuantity)
setattr(de, "hat_Größenart", M4I.hasKindOfQuantity)
setattr(M4I, "has_parameter", M4I.hasParameter)
setattr(de, "hat_Parameter", M4I.hasParameter)
setattr(M4I, "has_runtime_assignment", M4I.hasRuntimeAssignment)
setattr(de, "hat_Laufzeitzuweisung", M4I.hasRuntimeAssignment)
setattr(M4I, "has_uncertainty_declaration", M4I.hasUncertaintyDeclaration)
setattr(de, "hat_Unsicherheitsdeklaration", M4I.hasUncertaintyDeclaration)
setattr(M4I, "has_unit", M4I.hasUnit)
setattr(de, "hat_Einheit", M4I.hasUnit)
setattr(M4I, "has_variable", M4I.hasVariable)
setattr(de, "hat_Variable", M4I.hasVariable)
setattr(M4I, "associated_to_project", M4I.inProject)
setattr(de, "gehört_zu_Projekt", M4I.inProject)
setattr(M4I, "investigates", M4I.investigates)
setattr(de, "untersucht", M4I.investigates)
setattr(M4I, "investigates_property", M4I.investigatesProperty)
setattr(de, "untersucht_Eigenschaft", M4I.investigatesProperty)
setattr(M4I, "is_employed_tool", M4I.isEmployedToolIn)
setattr(de, "ist_eingesetztes_Werkzeug", M4I.isEmployedToolIn)
setattr(de, "hat_Projektmitglied", M4I.projectParticipant)
setattr(M4I, "project_participant", M4I.projectParticipant)
setattr(de, "realisiert_Methode", M4I.realizesMethod)
setattr(M4I, "realizes_method", M4I.realizesMethod)
setattr(de, "Verwendungshinweis", M4I.UsageInstruction)
setattr(M4I, "usage_instruction", M4I.UsageInstruction)
setattr(de, "Projektenddatum", M4I.endOfProject)
setattr(M4I, "project_end_date", M4I.endOfProject)
setattr(M4I, "has_assignment_timestamp", M4I.hasAssignmentTimestamp)
setattr(de, "hat_Zuweisungszeitstempel", M4I.hasAssignmentTimestamp)
setattr(M4I, "has_date_assignment_created", M4I.hasDateAssignmentCreated)
setattr(de, "hat_Datumszuweisung_erzeugt", M4I.hasDateAssignmentCreated)
setattr(M4I, "has_date_assignment_deleted", M4I.hasDateAssignmentDeleted)
setattr(de, "hat_Datumszuweisung_gelöscht", M4I.hasDateAssignmentDeleted)
setattr(M4I, "has_date_assignment_modified", M4I.hasDateAssignmentModified)
setattr(de, "hat_Datumszuweisung_bearbeitet", M4I.hasDateAssignmentModified)
setattr(M4I, "has_date_assignment_valid_from", M4I.hasDateAssignmentValidFrom)
setattr(de, "hat_Datumszuweisung_gültig_ab", M4I.hasDateAssignmentValidFrom)
setattr(M4I, "has_date_assignment_valid_until", M4I.hasDateAssignmentValidUntil)
setattr(de, "hat_Datumszuweisung_gültig_bis", M4I.hasDateAssignmentValidUntil)
setattr(M4I, "has_maximum_value", M4I.hasMaximumValue)
setattr(de, "hat_Maximalwert", M4I.hasMaximumValue)
setattr(M4I, "has_minimum_value", M4I.hasMinimumValue)
setattr(de, "hat_Minimalwert", M4I.hasMinimumValue)
setattr(M4I, "has_numerical_value", M4I.hasNumericalValue)
setattr(de, "hat_Zahlenwert", M4I.hasNumericalValue)
setattr(M4I, "has_ROR_ID", M4I.hasRorId)
setattr(M4I, "hat_ROR_ID", M4I.hasRorId)
setattr(M4I, "has_step_size", M4I.hasStepSize)
setattr(de, "hat_Schrittweite", M4I.hasStepSize)
setattr(M4I, "has_string_value", M4I.hasStringValue)
setattr(de, "hat_Zeichenwert", M4I.hasStringValue)
setattr(M4I, "has_symbol", M4I.hasSymbol)
setattr(de, "hat_Symbol", M4I.hasSymbol)
setattr(M4I, "has_value", M4I.hasValue)
setattr(de, "hat_Wert", M4I.hasValue)
setattr(M4I, "has_variable_description", M4I.hasVariableDescription)
setattr(de, "hat_Variablenbeschreibung", M4I.hasVariableDescription)
setattr(M4I, "has_identifier", M4I.identifier)
setattr(de, "hat_Identifikator", M4I.identifier)
setattr(M4I, "has_ORCID_ID", M4I.orcidId)
setattr(de, "hat_ORCID_ID", M4I.orcidId)
setattr(M4I, "has_project_ID", M4I.projectReferenceID)
setattr(de, "hat_Projekt-ID", M4I.projectReferenceID)
setattr(de, "Projektstartdatum", M4I.startOfProject)
setattr(M4I, "project_start_date", M4I.startOfProject)
setattr(de, "Kontaktperson", M4I.ContactPerson)
setattr(M4I, "contact_person", M4I.ContactPerson)
setattr(de, "Datenerfasser*in", M4I.DataCollector)
setattr(M4I, "data_collector", M4I.DataCollector)
setattr(de, "Datenkurator*in", M4I.DataCurator)
setattr(M4I, "data_curator", M4I.DataCurator)
setattr(de, "Datenverwalter*in", M4I.DataManager)
setattr(M4I, "data_manager", M4I.DataManager)
setattr(de, "Anbieter*in", M4I.Distributor)
setattr(M4I, "distributor", M4I.Distributor)
setattr(de, "Herausgeber*in", M4I.Editor)
setattr(M4I, "editor", M4I.Editor)
setattr(de, "bereitstellende_Institution", M4I.HostingInstitution)
setattr(M4I, "hosting_institution", M4I.HostingInstitution)
setattr(M4I, "other_person", M4I.Other)
setattr(de, "weitere_Person", M4I.Other)
setattr(de, "Produzent*in", M4I.Producer)
setattr(M4I, "producer", M4I.Producer)
setattr(de, "Projektleiter*in", M4I.ProjectLeader)
setattr(M4I, "project_leader", M4I.ProjectLeader)
setattr(de, "Projektmanager*in", M4I.ProjectManager)
setattr(M4I, "project_manager", M4I.ProjectManager)
setattr(de, "Projektmitglied", M4I.ProjectMember)
setattr(M4I, "project_member", M4I.ProjectMember)
setattr(de, "Registrierungsstelle", M4I.RegistrationAgency)
setattr(M4I, "registration_agency", M4I.RegistrationAgency)
setattr(de, "Registrierungsbehörde", M4I.RegistrationAuthority)
setattr(M4I, "registration_authority", M4I.RegistrationAuthority)
setattr(M4I, "related_person", M4I.RelatedPerson)
setattr(de, "zugehörige_Person", M4I.RelatedPerson)
setattr(de, "Forschungsgruppe", M4I.ResearchGroup)
setattr(M4I, "research_group", M4I.ResearchGroup)
setattr(de, "Rechercheur*in", M4I.Researcher)
setattr(M4I, "researcher", M4I.Researcher)
setattr(de, "Rechteinhaber*in", M4I.RightsHolder)
setattr(M4I, "rights_holder", M4I.RightsHolder)
setattr(de, "Sponsor*in", M4I.Sponsor)
setattr(M4I, "sponsor", M4I.Sponsor)
setattr(de, "Betreuer*in", M4I.Supervisor)
setattr(M4I, "supervisor", M4I.Supervisor)
setattr(de, "Arbeitspaketleiter*in", M4I.WorkPackageLeader)
setattr(M4I, "work_package_leader", M4I.WorkPackageLeader)

setattr(M4I, "de", de)


from ._version import __version__
from .m4i import M4I
from .obo import OBO
from .qudt_unit import QUDT_UNIT
from .qudt_kind import QUDT_KIND
from .pivmeta import PIVMETA
from .codemeta import CODEMETA
from .schema import SCHEMA
from .ssno import SSNO



import rdflib
import unittest

import namespacelib


class TestNamespaces(unittest.TestCase):

    def test_m4i(self):
        self.assertIsInstance(namespacelib.M4I.Tool, rdflib.URIRef)
        self.assertEqual(str(namespacelib.M4I.Tool),
                         'http://w3id.org/nfdi4ing/metadata4ing#Tool')

        with self.assertRaises(AttributeError):
            namespacelib.M4I.Invalid

    def test_qudt_unit(self):
        self.assertIsInstance(namespacelib.QUDT_UNIT.M_PER_SEC, rdflib.URIRef)
        self.assertEqual(str(namespacelib.QUDT_UNIT.M_PER_SEC),
                         'http://qudt.org/vocab/unit/M-PER-SEC')
        with self.assertRaises(AttributeError):
            namespacelib.QUDT_UNIT.METER

    def test_qudt_kind(self):
        self.assertIsInstance(namespacelib.QUDT_KIND.Mass, rdflib.URIRef)
        self.assertEqual(str(namespacelib.QUDT_KIND.Mass),
                         'http://qudt.org/vocab/quantitykind/Mass')

    def test_rdflib(self):
        self.assertIsInstance(rdflib.PROV.Agent, rdflib.URIRef)
        self.assertEqual(str(rdflib.PROV.Agent),
                         "http://www.w3.org/ns/prov#Agent")

    def test_codemeta(self):
        self.assertIsInstance(namespacelib.CODEMETA.softwareSuggestions, rdflib.URIRef)
        self.assertEqual(str(namespacelib.CODEMETA.softwareSuggestions),
                         "https://codemeta.github.io/terms/softwareSuggestions")

    def test_schema(self):
        self.assertIsInstance(namespacelib.SCHEMA.Person, rdflib.URIRef)
        self.assertEqual(str(namespacelib.SCHEMA.Person),
                         "https://schema.org/Person")

    def test_ssno(self):
        self.assertIsInstance(namespacelib.SSNO.StandardName, rdflib.URIRef)
        self.assertEqual(str(namespacelib.SSNO.StandardName),
                         "https://matthiasprobst.github.io/ssno#StandardName")
        with self.assertRaises(AttributeError):
            namespacelib.SSNO.METER



import json
import pathlib
import unittest

from namespacelib import __version__


def get_package_meta():
    """Reads codemeta.json and returns it as dict"""
    with open(__this_dir__ / '../codemeta.json', 'r') as f:
        codemeta = json.loads(f.read())
    return codemeta


__this_dir__ = pathlib.Path(__file__).parent


class TestVersion(unittest.TestCase):

    def test_version(self):
        this_version = 'x.x.x'
        setupcfg_filename = __this_dir__ / '../setup.cfg'
        with open(setupcfg_filename, 'r') as f:
            lines = f.readlines()
            for line in lines:
                if 'version' in line:
                    this_version = line.split(' = ')[-1].strip()
        self.assertEqual(__version__, this_version)

    def test_codemeta(self):
        """checking if the version in codemeta.json is the same as the one of the toolbox"""

        codemeta = get_package_meta()

        assert codemeta['version'] == __version__
